import 'dart:io';
import 'package:path/path.dart' as p;
import 'package:io/ansi.dart' as ansi;

const stopFolderList = const ['node_modules', '.dart_tool', 'build', 'site-packages'];

Future main() async {
    final userName = await getGitUserName();
    final messageList = await getGitRoots().asyncMap((Directory dir) async {
        final commits = await  runGitStandupCommand(dir,userName);

        if(commits.trim() == ""){
            return '';
        }
        return "${ansi.wrapWith(dir.path, [ansi.styleBold, ansi.red])}:\n\n${commits.trim()}\n\n";

    }).toSet();
    stdout.write(messageList.join(''));
}

Stream<Directory> getGitRoots([Directory dir]) async* {
  dir ??= Directory.current;

  if (!await isGitRepoRoot(dir.path)) {
    await for (var d in dir.list().where((d) => d is Directory)) {
      if (!shouldProceed(dir.path)) {
        break;
      }
      yield* getGitRoots(d);
    }
  } else {
    yield dir;
  }
}

bool shouldProceed(String path){
    return !isHiddenFile(path) && !stopFolderList.contains(p.basename(path));
}

bool isHiddenFile(String path) {
    return p.basename(path).startsWith('.');
}

Future<bool> isGitRepoRoot(String path) async {
  final result = (await Process.run(
    'git',
    ['rev-parse', '--show-toplevel'],
    workingDirectory: path,
  ))
      .stdout
      .toString()
      .split('\n')
      .first;

  return result == path.replaceAll('\n', '');
}

Future<String> getGitUserName() async {
  return (await Process.run('git', 'config user.name'.split(' '))).stdout;
}

Future<String> runGitStandupCommand(
  Directory dir,
  String username, [
  timeAgo = '1 day ago',
]) async {
  return (await Process.run(
    'git',
    [
      'log',
      '--author=$username',
      '--after="$timeAgo"',
      '--pretty=tformat:%s\n%b\r',
    ],
    workingDirectory: dir.path
  ))
      .stdout
      .toString();
}
